package com.huatek.frame.base.service;

import java.util.List;
import java.util.Map;

import com.huatek.frame.base.dto.RoleDTO;
import com.huatek.frame.base.entity.Role;
import com.huatek.frame.base.vo.RoleVO;

/**
 * 系统_角色 Service
 *
 * @author key_dong
 * @date 2018-7-11 14:03:46
 */
public interface RoleService {

	/**
	 * 分页查找查找 系统_角色
	 *
	 * @param page
	 *            页码
	 * @param limit
	 *            每页显示数量
	 * @param dto
	 *            系统_角色dto
	 * @return map map
	 */
	Map<String, Object> findRolePage(RoleDTO dto);

	/**
	 * 添加 系统_角色
	 * 
	 * @param role
	 *            系统_角色role
	 * @return map map
	 */
	Map<String, Object> addRole(Role role);

	/**
	 * 通过id查找 系统_角色
	 *
	 * @param id
	 *            主键
	 * @return map map
	 */
	Map<String, Object> findRole(String id);

	/**
	 * 修改 系统_角色
	 * 
	 * @param role
	 *            系统_角色role
	 * @return map map
	 */
	Map<String, Object> updateRole(Role role);

	/**
	 * 删除 系统_角色
	 * 
	 * @param ids 主键集合
	 * @return map map
	 */
	Map<String, Object> deltetRole(String[] ids);

	/**
	 * 修改角色菜单
	 * 
	 * @param pIds
	 *            资源id集合
	 * @param rId
	 *            角色id
	 * @return
	 */
	Map<String, Object> updateRolePermission(String[] pIds, String rId);

	/**
	 * 角色集合接口
	 * @param uid 
	 * @param token 
	 * 
	 * @return
	 */
	Map<String, Object> findRoles(String uid, String token);
	
	/**
	 * 查询所有正常的角色
	 * @return
	 */
	List<Role> findRoles();
	
	/**
	 * 根据用户查询角色集合
	 * @param uid
	 * @return
	 */
	List<RoleVO> findRoles(String uid);
}
